@@ -99,23 +99,49 @@ char *errorMessage = NULL;
 
 static
 void
-WRMT_ClearAllInputBufferReportID()
+WRMT_WiiRemote_InitWithDeviceIndex(WRMT_WiiRemote *self,
+	int device_index)
 {
-	int i, number;
-	WRMT_WiiRemote *wiiRemote;
+	int i;
+	WRMT_WiiRemoteImpl *wiiRemoteImpl;
+	assert(self != NULL);
+	assert(device_index >= 0 && device_index <= WRMT_MAX_DEVICES);
 	DebugLog("Hello\n");
 
-	/* Clear report ID */
-	number = WRMT_Impl_GetNumWiiRemote();
-	for (i = 0;i < number;i++)
+	wiiRemoteImpl = WRMT_Impl_GetWiiRemoteAt(device_index);
+	self->impl = wiiRemoteImpl;
+	self->device_index = device_index;
+	self->output_buffer = WRMT_WiiRemoteImpl_GetOutputBuffer(wiiRemoteImpl);
+	self->input_buffer = WRMT_WiiRemoteImpl_GetInputBuffer(wiiRemoteImpl);
+
+	for (i = 0;i < WRMT_NUMBER_OF_FUNCTIONS;i++)
 	{
-		wiiRemote = &(wiiRemoteList[i]);
-		wiiRemote->input_buffer[0] = 0;
+		self->function_flag[i] = 0;
 	}
+	for (i = 0;i < WRMT_NUMBER_OF_DATA_TYPE;i++)
+	{
+		self->device_state[i] = 0;
+	}
+	self->device_state[WRMT_DATA_SPEAKER_VOLUME] = DEFAULT_SPEAKER_VOLUME;
+	self->device_state[WRMT_DATA_SPEAKER_SAMPLE_RATE] = DEFAULT_SPEAKER_SAMPLE_RATE;
 
 	DebugLog("Bye\n");
+	WRMT_WiiRemote_Invaliant(self);
 }
 
+static
+void
+WRMT_WiiRemote_ParseBatteryLevel(WRMT_WiiRemote *self)
+{
+	assert(self->input_buffer[0] == 0x20);
+	WRMT_WiiRemote_Invaliant(self);
+	DebugLog("Hello\n");
+
+	self->device_state[WRMT_DATA_BATTERY_LEVEL] = self->input_buffer[6];
+
+	DebugLog("Bye\n");
+	WRMT_WiiRemote_Invaliant(self);
+}
 
 static
 void
@@ -229,32 +255,50 @@ WRMT_WiiRemote_ParseIRState(WRMT_WiiRemote *self)
 		}
 	}
 
-	DebugLog("Hello\n");
+	DebugLog("Bye\n");
 	WRMT_WiiRemote_Invaliant(self);
 }
 
-
 static
 void
 WRMT_WiiRemote_ParseReport(WRMT_WiiRemote *self)
 {
+	unsigned char report_id;
 	WRMT_WiiRemote_Invaliant(self);
 	DebugLog("Hello\n");
 
-	if ((self->input_buffer[0] & 0xf0) == 0x30)
+#ifdef DUMP_INPUT_BUFFER
+	{
+		int i;
+		for (i = 0;i < WRMT_BUFFER_SIZE;i++)
+			fprintf(stderr, "%02x ", self->input_buffer[i]);
+		fprintf(stderr, "\n");
+	}
+#endif
+
+	report_id = self->input_buffer[0];
+	DebugLog("report_id : 0x%02x\n", report_id);
+	if ((report_id & 0xf0) == 0x30)
 	{
 		WRMT_WiiRemote_ParseButtonState(self);
-		if (self->input_buffer[0] & 0x01)
+		if (report_id & 0x01)
 		{
 			WRMT_WiiRemote_ParseMotionState(self);
 		}
 
-		if ((self->input_buffer[0] & 0x02) &&
+		if ((report_id & 0x02) &&
 			self->function_flag[WRMT_FUNCTION_IR])
 		{
 			WRMT_WiiRemote_ParseIRState(self);
 		}
 	}
+	else if (report_id == 0x20)
+	{ /* Expansion port and controler battery status */
+		WRMT_WiiRemote_ParseBatteryLevel(self);
+	}
+
+	/* clear report ID */
+	self->input_buffer[0] = 0;
 
 	DebugLog("Bye\n");
 	WRMT_WiiRemote_Invaliant(self);
@@ -336,6 +380,7 @@ WRMT_WiiRemote_SendReportTypeRequest(WRMT_WiiRemote *self)
 	if (self->function_flag[WRMT_FUNCTION_IR])
 		channel |= 0x02;
 
+	DebugLog("channel : 0x%02x\n", channel);
 	WRMT_WiiRemote_ClearOutputBuffer(self);
 	self->output_buffer[0] = WRMT_OUTPUT_REPORT_ID_REQUEST_REPORT_TYPE;
 	self->output_buffer[1] = continuous | force_feedback;
@@ -394,6 +439,7 @@ WRMT_WiiRemote_SendIRParameters(WRMT_WiiRemote *self)
 	self->output_buffer[1] = ir_flag | force_feedback_flag;
 	result = WRMT_WiiRemote_OutputToDevice(self);
 	if (result != WRMT_IO_SUCCESS) return result;
+	WRMT_Impl_Sleep(10); /* ?? */
 
 	/* IR Sensor Enable 2 */
 	WRMT_WiiRemote_ClearOutputBuffer(self);
@@ -401,6 +447,7 @@ WRMT_WiiRemote_SendIRParameters(WRMT_WiiRemote *self)
 	self->output_buffer[1] = ir_flag | force_feedback_flag;
 	result = WRMT_WiiRemote_OutputToDevice(self);
 	if (result != WRMT_IO_SUCCESS) return result;
+	WRMT_Impl_Sleep(10); /* ?? */
 
 	if (self->function_flag[WRMT_FUNCTION_IR])
 	{
@@ -411,30 +458,30 @@ WRMT_WiiRemote_SendIRParameters(WRMT_WiiRemote *self)
 		result = WRMT_WiiRemote_WriteDataToMemory(self, 
 			0x04b00030, &disable_ir, 1);
 		if (result != WRMT_IO_SUCCESS) return result;
-		WRMT_Impl_Sleep(10);
+		WRMT_Impl_Sleep(10); /* ?? */
 
 		/* set ir sensitivity */
 		result = WRMT_WiiRemote_WriteDataToMemory(self, 
 			0x04b00000, IR_SENSITIVITY_MIDRANGE_DATA_1, 9);
 		if (result != WRMT_IO_SUCCESS) return result;
-		WRMT_Impl_Sleep(10);
+		WRMT_Impl_Sleep(10); /* ?? */
 
 		result = WRMT_WiiRemote_WriteDataToMemory(self, 
 			0x04b0001a, IR_SENSITIVITY_MIDRANGE_DATA_2, 2);
 		if (result != WRMT_IO_SUCCESS) return result;
-		WRMT_Impl_Sleep(10);
+		WRMT_Impl_Sleep(10); /* ?? */
 
 		/* set ir mode */
 		result = WRMT_WiiRemote_WriteDataToMemory(self, 
 			0x04b00033, &ir_mode, 1);
 		if (result != WRMT_IO_SUCCESS) return result;
-		WRMT_Impl_Sleep(10);
+		WRMT_Impl_Sleep(10); /* ?? */
 
 		/* start ir camera */
 		result = WRMT_WiiRemote_WriteDataToMemory(self, 
 			0x04b00030, &enable_ir, 1);
 		if (result != WRMT_IO_SUCCESS) return result;
-		WRMT_Impl_Sleep(10);
+		WRMT_Impl_Sleep(10); /* ?? */
 	}
 
 	DebugLog("Bye\n");
@@ -541,37 +588,6 @@ WRMT_GetError()
 	return (char *)errorMessage;
 }
 
-void
-WRMT_WiiRemote_InitWithDeviceIndex(WRMT_WiiRemote *self,
-	int device_index)
-{
-	int i;
-	WRMT_WiiRemoteImpl *wiiRemoteImpl;
-	assert(self != NULL);
-	assert(device_index >= 0 && device_index <= WRMT_MAX_DEVICES);
-	DebugLog("Hello\n");
-
-	wiiRemoteImpl = WRMT_Impl_GetWiiRemoteAt(device_index);
-	self->impl = wiiRemoteImpl;
-	self->device_index = device_index;
-	self->output_buffer = WRMT_WiiRemoteImpl_GetOutputBuffer(wiiRemoteImpl);
-	self->input_buffer = WRMT_WiiRemoteImpl_GetInputBuffer(wiiRemoteImpl);
-
-	for (i = 0;i < WRMT_NUMBER_OF_FUNCTIONS;i++)
-	{
-		self->function_flag[i] = 0;
-	}
-	for (i = 0;i < WRMT_NUMBER_OF_DATA_TYPE;i++)
-	{
-		self->device_state[i] = 0;
-	}
-	self->device_state[WRMT_DATA_SPEAKER_VOLUME] = DEFAULT_SPEAKER_VOLUME;
-	self->device_state[WRMT_DATA_SPEAKER_SAMPLE_RATE] = DEFAULT_SPEAKER_SAMPLE_RATE;
-
-	DebugLog("Bye\n");
-	WRMT_WiiRemote_Invaliant(self);
-}
-
 int
 WRMT_Init()
 {
@@ -631,9 +647,6 @@ WRMT_Poll(int *updated_device_index_pointer)
 	int updated_device_index;
 	DebugLog("Hello\n");
 
-	/* Clear report ID */
-	WRMT_ClearAllInputBufferReportID();
-
 	/* Polling devices*/
 	result = WRMT_Impl_Poll(&updated_device_index);
 
@@ -684,9 +697,11 @@ WRMT_WiiRemote_Open(WRMT_WiiRemote *self)
 	result = WRMT_WiiRemoteImpl_Open(self->impl);
 	if (result == WRMT_IO_SUCCESS)
 	{
-		WRMT_WiiRemote_SendControllerStatusRequest(self);
 		WRMT_WiiRemote_SendReportTypeRequest(self);
+		WRMT_Sleep(10);
 		WRMT_WiiRemote_SendDeviceState(self);
+		WRMT_Sleep(10);
+		WRMT_WiiRemote_SendControllerStatusRequest(self);
 	}
 
 	DebugLog("Bye\n");
@@ -763,6 +778,7 @@ WRMT_WiiRemote_SetEnabled(WRMT_WiiRemote *self, WRMT_FunctionType type, int valu
 			break;
 		}
 	}
+	WRMT_Impl_Sleep(10);
 
 	DebugLog("Bye\n");
 	WRMT_WiiRemote_Invaliant(self);
